
#ifndef LDFS_COMMON_H_
#define LDFS_COMMON_H_
#include <linux/module.h>
#include <linux/fs.h>
#include <crypto/hash.h>
#include "ext4.h"

#define LDFS_READ_FIX
#ifdef LDFS_READ_FIX
#define LDFS_READ_CACHE_ENABLE
#define LDFS_READ_REORDER_ENABLE
#endif
#ifdef LDFS_READ_CACHE_ENABLE
#define DDFS_READ_CACHE_MAX (256*1024)
#endif
#define LDFS_PARAL_WRITE



struct ldfs_fp_info {
	char * name;
	unsigned long len;
	char * zero_result;
	int (*update)(struct shash_desc *shash, const u8 *data, unsigned int len);
};


void ldfs_prev_init(struct super_block * sb);
int ldfs_prev_exist(struct super_block * sb, __u8 fp[]);
int ldfs_prev_insert(struct super_block * sb, struct ldfs_blk_info * i);

int ldfs_fp_info_assign(struct super_block * sb);
int ldfs_fp_compute(struct super_block * sb, __u8 dst[], char * src, int len);
int ldfs_fp_equal(struct super_block * sb, __u8 * fp1, __u8 * fp2);
int ldfs_fp_zero(struct super_block * sb, __u8 * fp);
void ldfs_fp_print(struct super_block * sb, __u8 * fp);




int ldfs_workqueue_init(struct super_block * sb,int thread_no);
void ldfs_workqueue_exit(struct super_block * sb);

int ldfs_fingerprint_enqueue(struct super_block * sb, struct ldfs_job_info * info, int cpu);

void ldfs_fingerprint_compute_batch(struct super_block * sb,int len, struct ldfs_blk_info * b_info);

#endif /* LDFS_COMMON_H_ */
