
#ifndef _LDFS_FILTER_H
#define _LDFS_FILTER_H

#include <linux/module.h>
#include <linux/string.h>
#include <linux/fs.h>
#include <linux/slab.h>
#include "ext4.h"
#include "ldfs_slist.h"

/*
 * Test only
 * Excludes Locality Cache
 * So that deduplication must be exact
 */
#define LDFS_FILTER_TEST
#undef LDFS_FILTER_TEST

#define LDFS_BUCKET_SIZE 256

struct ldfs_part_fp {
	__u8 data[3];
};

/*
 * This implementation only supports 32-bit file-system
 * However current version of mkfs.ext4 also only supports 32-bit
 */
#ifndef LDFS_FILTER_TEST
#define LDFS_BUCKET_ENTRY_MAX 49

struct ldfs_bucket {
	struct slist_head b_head;
	__u16 b_count;
	__u16 b_part_blocknr[LDFS_BUCKET_ENTRY_MAX];
	struct ldfs_part_fp b_part_fp[LDFS_BUCKET_ENTRY_MAX];
	__u8 b_padding;
};
#else
#define LDFS_BUCKET_ENTRY_MAX 35

struct ldfs_bucket {
	struct slist_head b_head;
	__u16 b_count;
	__u8 b_padding;
	struct ldfs_part_fp b_part_fp[LDFS_BUCKET_ENTRY_MAX];
	__u32 b_part_blocknr[LDFS_BUCKET_ENTRY_MAX];
};
#endif



int ldfs_filter_init(struct super_block * sb);
int ldfs_filter_exit(struct super_block * sb);
int ldfs_filter_search(struct super_block * sb, __u8 fp[], ext4_fsblk_t * blocknr);
int ldfs_filter_insert_entry(struct super_block * sb, __u8 fp[], ext4_fsblk_t blocknr);
int ldfs_filter_remove_entry(struct super_block * sb, __u8 fp[], ext4_fsblk_t blocknr);

int ldfs_fingerprint_search(struct super_block * sb, __u8 fp[], ext4_fsblk_t * blocknr);
int ldfs_fingerprint_insert(struct super_block * sb, __u8 fp[], ext4_fsblk_t blocknr);
int ldfs_fingerprint_remove(struct super_block * sb, __u8 fp[], ext4_fsblk_t blocknr);

#endif /* _LDFS_FILTER_H */
